use core::{ffi::c_void, pin::Pin};

use alloc::boxed::Box;
use flipperzero_sys as sys;

use crate::{scenes::SCENE_EVENT_HANDLERS, views::view_dispatcher_init};

pub struct App {
    pub view_dispatcher: *mut sys::ViewDispatcher,
    pub scene_manager: Option<*mut sys::SceneManager>,
    pub gui: sys::furi::UnsafeRecord<sys::Gui>,
    pub menu: *mut sys::Menu,
    pub popup: *mut sys::Popup
}

impl App {
    pub fn new() -> Pin<Box<Self>> {
        let mut app = Box::pin(App {
            view_dispatcher: unsafe { sys::view_dispatcher_alloc() },
            scene_manager: None,
            gui: unsafe { sys::furi::UnsafeRecord::open(c"gui".as_ptr()) as sys::furi::UnsafeRecord<sys::Gui> },
            menu: unsafe { sys::menu_alloc() },
            popup: unsafe { sys::popup_alloc() }
        });
        app.scene_manager = unsafe { Some(sys::scene_manager_alloc(&SCENE_EVENT_HANDLERS, &*app as *const App as *mut c_void)) };
        unsafe { view_dispatcher_init(&mut app); }
        app
    }
}

impl Drop for App {
    fn drop(&mut self) {
        unsafe {
            sys::view_dispatcher_free(self.view_dispatcher);
            sys::scene_manager_free(self.scene_manager.unwrap());
        }
    }
}
