//! Template project for Flipper Zero.
//! This app prints "Hello, Rust!" to the console then exits.

#![no_main]
#![no_std]
#![feature(variant_count)]
#![feature(const_refs_to_static)]

extern crate alloc;
extern crate flipperzero_alloc;
extern crate flipperzero_rt;

#[macro_use]
extern crate enum_primitive_derive;
extern crate num_traits;

use core::ffi::CStr;

use flipperzero_rt::{entry, manifest};
use flipperzero_sys as sys;
use scenes::Scenes;
use app::App;
use views::Views;

// Define the FAP Manifest for this application
manifest!(
    name = "Flipper Hewo World in Rust",
    app_version = 1,
    has_icon = true,
    // See https://github.com/flipperzero-rs/flipperzero/blob/v0.7.2/docs/icons.md for icon format
    icon = "icons/rustacean-10x10.icon",
);

// Define the entry function
entry!(main);

mod main_menu;
mod popup_one;
mod popup_two;
mod scenes;
mod views;
mod app;

// Entry point
fn main(_args: Option<&CStr>) -> i32 {
    let app = App::new();

    unsafe {
        sys::view_dispatcher_attach_to_gui(app.view_dispatcher, app.gui.as_ptr(), sys::ViewDispatcherType_ViewDispatcherTypeFullscreen);
        sys::scene_manager_next_scene(app.scene_manager.unwrap(), Scenes::MainMenu as u32);
        sys::view_dispatcher_run(app.view_dispatcher);
    }

    // Avoid app to be drop befoer the end of the process
    drop(app);

    0
}
