use core::{ffi::c_void, ptr::null};

use flipperzero_sys as sys;
use num_traits::FromPrimitive;
use sys::SceneManagerEvent;

use crate::{app::App, scenes::Scenes, views::{app_exit, Views}};

#[derive(Primitive)]
enum Menus {
    SelectionOne = 0,
    SelectionTwo = 1
}

#[derive(Primitive)]
enum Events {
    MenuShowPopupOne = 0,
    MenuShowPopupTwo = 1
}

pub unsafe extern "C" fn menu_callback(context: *mut c_void, index: u32) {
    let app = context as *mut App;

    match Menus::from_u32(index) {
        Some(Menus::SelectionOne) => sys::scene_manager_handle_custom_event((*app).scene_manager.unwrap(), Events::MenuShowPopupOne as u32),
        Some(Menus::SelectionTwo) => sys::scene_manager_handle_custom_event((*app).scene_manager.unwrap(), Events::MenuShowPopupTwo as u32),
        _ => return,
    };
}

pub unsafe extern "C" fn scene_on_enter(context: *mut c_void) {
    let app = context as *mut App;
    sys::menu_reset((*app).menu);
    sys::menu_add_item((*app).menu, c"First popup".as_ptr(), null(), Menus::SelectionOne as u32, Some(menu_callback), context);
    sys::menu_add_item((*app).menu, c"Second popup".as_ptr(), null(), Menus::SelectionTwo as u32, Some(menu_callback), context);
    sys::view_set_previous_callback(sys::menu_get_view((*app).menu), Some(app_exit));
    sys::view_dispatcher_switch_to_view((*app).view_dispatcher, Views::Menu as u32);
}

pub unsafe extern "C" fn scene_on_event(context: *mut c_void, event: SceneManagerEvent) -> bool {
    let app = context as *mut App;

    match Events::from_u32(event.event) {
        Some(Events::MenuShowPopupOne) => {sys::scene_manager_next_scene((*app).scene_manager.unwrap(), Scenes::PopupOne as u32); true},
        Some(Events::MenuShowPopupTwo) => {sys::scene_manager_next_scene((*app).scene_manager.unwrap(), Scenes::PopupTwo as u32); true},
        _ => false,
    }
}

pub unsafe extern "C" fn scene_on_exit(context: *mut c_void) {
    let app = context as *mut App;
    sys::menu_reset((*app).menu);
}
